#ifndef _SIMPLICIAL_COMPLEX_EDGE_H_
#define _SIMPLICIAL_COMPLEX_EDGE_H_


//////////////////////////////////////////////////////////////////////////////
// Includes

#include "util/linear.h"


//////////////////////////////////////////////////////////////////////////////

template<int N>
class SimplicialComplex;
class Vertex;


//////////////////////////////////////////////////////////////////////////////
// class Edge
// A class for doing operations on and querying a 1-simplex of a complex

class Edge
{
    // friends
    template<typename T>
    friend class SimplexIterator;
    friend class VertexEdgeIterator;
    friend class EdgeFaceIterator;
    friend class EdgeTetIterator;
    friend class SimplicialComplex;
    friend class Face;

    friend bool operator==( const Edge&, const Edge& );
    friend bool operator!=( const Edge&, const Edge& );

public:  // methods
    Edge();
    ~Edge();

    operator bool() const;

    size_t Index( void ) const  { return index; }

    vec3 Center  ( void ) const;
    real Length  ( void ) const;
    real DualArea( void ) const;

    void Vertices( vec3& v0, vec3& v1 ) const;
    void Vertices( Vertex& v0, Vertex& v1 ) const;
    EdgeFaceIterator Faces( void ) const;
    EdgeTetIterator Tets( void ) const;

    bool IsOnBoundary( void ) const;

private:  // methods
    Edge( const SimplicialComplex<3> *sc, size_t index );

private:  // data
    const SimplicialComplex<3> *sc;
    size_t index;
};


#endif  // _SIMPLICIAL_COMPLEX_EDGE_H_
