#include "simplicialComplex/io/io.h"
#include "simplicialComplex/simplicialComplex.h"
#include "simplicialComplex/io/tetFile.h"
#include "util/filename.h"
#include "util/default_logs.h"
#include <fstream>

using namespace std;


FileType GetFileType( const char* filename )
{
    string extension = Filename(filename).GetExtension();

    if( extension == "obj" )
        return OBJ;
    else if( extension == "off" )
        return OFF;
    else if( extension == "tet" )
        return TET;
    else
        LOG(err) << "Unrecognized mesh file type";
    return AUTO;
}

//bool Read( SimplicialComplex<2>& mesh, std::istream& in, FileType type )
//{
//    switch( type )
//    {
//    case OBJ:
//        return OBJ_File::Read(mesh, in);
//
//    case OFF:
//        return OFF_File::Read(mesh, in);
//
//    case TET:
//        LOG(err) << "This is not a 2d mesh file";
//        return false;
//
//    default:
//        LOG(err) << "Unrecognized mesh file type";
//        return false;
//    }
//}

bool Read( SimplicialComplex<3>& mesh, std::istream& in, FileType type )
{
    switch( type )
    {
    case TET:
        return TET_File::Read(mesh, in);

    case OBJ:
        LOG(err) << "This is not a tet mesh file";
        return false;

    case OFF:
        LOG(err) << "This is not a tet mesh file";
        return false;

    default:
        LOG(err) << "Unrecognized mesh file type";
        return false;
    }
}

template<int N>
bool Read( SimplicialComplex<N>& mesh, const char* filename, FileType type )
{
    if( type == AUTO )
    {
        type = GetFileType( filename );
        if( type == AUTO )
            return false;
    }

    std::ifstream in(filename);
    if( !in )
    {
        LOG(err) << "Couldn't open " << filename << " for reading";
        return false;
    }

    LOG(info) << "Reading mesh file " << filename;
    return Read( mesh, in, type );
}
