//////////////////////////////////////////////////////////////////////////////
// simplicialComplex.cpp
// 2d and 3d meshes
//////////////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////////////////
// Includes

#include "simplicialComplex/simplicialComplex.h"
#include "simplicialComplex/iterator.h"
#include "simplicialComplex/vertex.h"
#include "simplicialComplex/edge.h"
#include "simplicialComplex/face.h"
#include "simplicialComplex/tet.h"
#include "util/assert.h"


//////////////////////////////////////////////////////////////////////////////
// Default Constructor

SimplicialComplex<3>::SimplicialComplex()
{
    // empty
}

//////////////////////////////////////////////////////////////////////////////
// Destructor

SimplicialComplex<3>::~SimplicialComplex()
{
    // empty
}

//////////////////////////////////////////////////////////////////////////////
// Vertices

SimplexIterator<Vertex> SimplicialComplex<3>::Vertices( void ) const
{
    return SimplexIterator<Vertex>(this);
}

//////////////////////////////////////////////////////////////////////////////
// Edges

SimplexIterator<Edge> SimplicialComplex<3>::Edges( void ) const
{
    return SimplexIterator<Edge>(this);
}

//////////////////////////////////////////////////////////////////////////////
// Faces

SimplexIterator<Face> SimplicialComplex<3>::Faces( void ) const
{
    return SimplexIterator<Face>(this);
}

//////////////////////////////////////////////////////////////////////////////
// Tets

SimplexIterator<Tet> SimplicialComplex<3>::Tets( void ) const
{
    return SimplexIterator<Tet>(this);
}

//////////////////////////////////////////////////////////////////////////////
// Get

Vertex SimplicialComplex<3>::GetVertex( size_t index ) const
{
    smart_assert( index < NumVerts() ).msg( "Invalid index" ).fatal();
    return Vertex( this, index );
}

//////////////////////////////////////////////////////////////////////////////
// Get

Edge SimplicialComplex<3>::GetEdge( size_t index ) const
{
    smart_assert( index < NumEdges() ).msg( "Invalid index" ).fatal();
    return Edge( this, index );
}

//////////////////////////////////////////////////////////////////////////////
// Get

Face SimplicialComplex<3>::GetFace( size_t index ) const
{
    smart_assert( index < NumFaces() ).msg( "Invalid index" ).fatal();
    return Face( this, index );
}

//////////////////////////////////////////////////////////////////////////////
// Get

Tet SimplicialComplex<3>::GetTet( size_t index ) const
{
    smart_assert( index < NumTets() ).msg( "Invalid index" ).fatal();
    return Tet( this, index );
}
