
//////////////////////////////////////////////////////////////////////////////
// Includes

#include "simplicialComplex/vertex.h"
#include "simplicialComplex/simplicialComplex.h"
#include "simplicialComplex/vertexEdgeIterator.h"
#include "util/assert.h"


//////////////////////////////////////////////////////////////////////////////
// Default Constructor

Vertex::Vertex() : sc(0), index(size_t(-1))
{
    // empty
}

//////////////////////////////////////////////////////////////////////////////
// Private Constructor that takes a mesh pointer and index

Vertex::Vertex( const SimplicialComplex<3> *sc, size_t index ) : sc(sc), index(index)
{
    // empty
}

//////////////////////////////////////////////////////////////////////////////
// Destructor

Vertex::~Vertex()
{
    // empty
}

//////////////////////////////////////////////////////////////////////////////
// operator bool - Is this face valid

Vertex::operator bool() const
{
    return (sc && index < sc->NumVerts());
}

//////////////////////////////////////////////////////////////////////////////
// Position

vec3 Vertex::Position() const
{
    smart_assert(*this).msg( "Invalid vertex" ).fatal();
    return sc->m_vertices[index].Center();
}

//////////////////////////////////////////////////////////////////////////////
// DualVolume

real Vertex::DualVolume() const
{
    smart_assert(*this).msg( "Invalid vertex" ).fatal();
    return sc->m_vertices[index].DualVolume();
}

//////////////////////////////////////////////////////////////////////////////
// Edges

VertexEdgeIterator Vertex::Edges( void ) const
{
    return VertexEdgeIterator(*this);
}

//////////////////////////////////////////////////////////////////////////////
// IsOnBoundary - Warning! Slow!

bool Vertex::IsOnBoundary() const
{
    smart_assert(*this).msg( "Invalid vertex" ).fatal();
    // It's a boundary vertex if any of its edges are boundary edges
    for( VertexEdgeIterator edge = Edges(); edge; ++edge )
        if( edge->IsOnBoundary() )
            return true;
    return false;
}

//////////////////////////////////////////////////////////////////////////////
// Equality operator

bool operator==( const Vertex& lhs, const Vertex& rhs )
{
    return (lhs.sc == rhs.sc && lhs.index == rhs.index);
}

//////////////////////////////////////////////////////////////////////////////
// Inequality operator

bool operator!=( const Vertex& lhs, const Vertex& rhs )
{
    return (!(lhs == rhs));
}
