#include "util/default_logs.h"
#include "util/log/functions.h"

DEFINE_LOG(dbg, "DEBUG")
DEFINE_LOG(info, "Info")
DEFINE_LOG(warn, "Warning")
DEFINE_LOG(err, "ERROR")

namespace ns_log {

void InitDefaultLogs()
{
    // all log messages are prefixed by the log name ("Info", "DEBUG", etc)
    add_modifier("*", &prepend_prefix);

    // all logs prefix the message by time
    add_modifier("*", &prepend_time);

    // newline after each message
    add_modifier("*", &append_enter);

    // all messages are written to cout and 'log.txt'
    add_appender("*", &write_to_cout);
    add_appender("*", write_to_file("log.txt", false) );

    // err messages are also written to 'errors.txt'
    add_appender("ERROR", write_to_file("errors.txt", true) );

#ifdef WIN32
    // and dbg messages are also written to the debug window
    add_appender("DEBUG", &write_to_dbg_wnd );
#endif

    // enable all logs
    enable_logs("*");
}

} //  namespace ns_log
