#ifndef _LOG_LOG_H_
#define _LOG_LOG_H_

#include <sstream>

namespace ns_log {

/*
A logger. It has a name, it can be enabled/disabled, it can have appenders and/or modifiers.
Multiple logs with the same name share the same impl.
*/
struct logger
{
    logger(const std::string & log_name);
    ~logger();

    std::string name() const;

    void enable( bool b );
    bool is_enabled() const;

    void write_msg(const std::string & msg);

private:
    struct logger_impl *m_impl;
};


struct logger_stream
{
    logger_stream(logger & l) : m_logger(l) {}

    ~logger_stream()
    {
        // the actual writing of the message
        m_logger.write_msg( m_stream.str() );
    }

    std::ostringstream & stream() { return m_stream; }

private:
    // no copying
    logger_stream(const logger_stream&);
    logger_stream & operator=(const logger_stream&) const;

    std::ostringstream m_stream;
    logger & m_logger;
};


}  // namespace ns_log



#define DECLARE_LOG(log_name) ::ns_log::logger & log_name##_log();

#define DEFINE_LOG(log_name,log_str) ::ns_log::logger & log_name##_log() { static ::ns_log::logger l(log_str); return l; }

#define LOG(log_name) if( !log_name##_log().is_enabled() ) ; else (::ns_log::logger_stream(log_name##_log()).stream())

#define IS_LOG_ENABLED(log_name) log_name##_log().is_enabled()

#define INIT_LOGS() ::ns_log::InitDefaultLogs()


#endif  // _LOG_LOG_H_
