#ifndef _LOG_LOG_IMPL_H_
#define _LOG_LOG_IMPL_H_

#include "util/log/log.h"
#include "util/function.h"
#include <sstream>
#include <vector>

namespace ns_log {

// logs a message
typedef function<void, const std::string&, const std::string&> appender_func;
// modifies a message (like adds a prefix and/or suffix)
typedef function<void, const std::string&, std::string&> modifier_func;


void add_appender(const std::string & str, appender_func f);
void add_modifier(const std::string & str, modifier_func f);

void enable_logs(const std::string & logs_spec);
void disable_logs(const std::string & logs_spec);


struct logger_impl
{
    friend struct logger;

    logger_impl(const std::string & name) : name(name), m_is_enabled(false) {}
    ~logger_impl() {}

    void enable(bool val)    { m_is_enabled = val; }
    bool is_enabled() const  { return m_is_enabled; }

    void add_appender(appender_func a)  { m_appenders.push_back(a); }
    void add_modifier(modifier_func m)  { m_modifiers.push_back(m); }

    const std::vector<appender_func> & appenders() const { return m_appenders; }
    const std::vector<modifier_func> & modifiers() const { return m_modifiers; }

    const std::string name;

private:
    logger_impl & operator=(const logger_impl&) const;

    std::vector<appender_func> m_appenders;
    std::vector<modifier_func> m_modifiers;
    bool m_is_enabled;
};


}  // namespace ns_log


#endif  // _LOG_LOG_IMPL_H_
